# @package      hubzero-submit-distributor
# @file         RemoteBatchFACTORY.py
# @author       Steven Clark <clarks@purdue.edu>
# @copyright    Copyright (c) 2012-2014 HUBzero Foundation, LLC.
# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
#
# Copyright (c) 2012-2014 HUBzero Foundation, LLC.
#
# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
#
# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
# software: you can redistribute it and/or modify it under the terms of
# the GNU Lesser General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# HUBzero is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# HUBzero is a registered trademark of HUBzero Foundation, LLC.
#

import re
import logging

class RemoteBatchFACTORY:
   def __init__(self,
                hubUserName,
                hubUserId,
                runName,
                localJobId,
                instanceId,
                instanceDirectory,
                appScriptName,
                environment,
                isMultiCoreRequest,
                siteInfo,
                managerInfo,
                wallTime):
      self.logger             = logging.getLogger(__name__)
      self.hubUserName        = hubUserName
      self.hubUserId          = hubUserId
      self.runName            = runName
      self.localJobId         = localJobId
      self.instanceId         = instanceId
      self.instanceDirectory  = instanceDirectory
      self.stageFiles         = siteInfo['stageFiles'] and not siteInfo['sharedUserSpace']
      self.appScriptName      = appScriptName
      self.environment        = environment
      self.isMultiCoreRequest = isMultiCoreRequest
      self.computationMode    = managerInfo['computationMode']
      self.wallTime           = str(int(wallTime))

      self.nodeFileName = ""
      self.nodeList     = []


   def __makeSerialNotStagedTemplate(self):
      return """# RemoteBatchFACTORY:makeSerialNotStagedTemplate
universe = vanilla
executable = INSTANCEDIRECTORY/APPSCRIPTNAME
transfer_executable = true
transfer_input_files = JOBID_INSTANCEID_input.tar.gz
transfer_output_files = JOBID_INSTANCEID_output.tar.gz
environment = "ENVIRONMENT"
output = INSTANCEDIRECTORY/factory_RUNNAME_INSTANCEID.stdout
error = INSTANCEDIRECTORY/factory_RUNNAME_INSTANCEID.stderr
log = INSTANCEDIRECTORY/factory_RUNNAME_INSTANCEID.log
should_transfer_files = YES
when_to_transfer_output = ON_EXIT
notification = never
+FactoryJob = true
#+DESIRED_Sites="CMS_T2_US_Nebraska_Husker,CMS_T2_US_Nebraska_Red"
#+DESIRED_Sites="HCC_US_Nebraska_pf"
#requirements = (GLIDEIN_Entry_Name == "CMS_T2_US_Nebraska_Husker") || (GLIDEIN_Entry_Name == "CMS_T2_US_Nebraska_Red")
#requirements = (GLIDEIN_Entry_Name == "HCC_US_Nebraska_pf")
#requirements = (GLIDEIN_Entry_Name == "CMS_T2_US_Nebraska_Husker")
x509userproxy = /tmp/hub-proxy.HUBUSERNAME
+FactoryRefreshProxy = true
periodic_release = (CurrentTime > (EnteredCurrentStatus + 300)) && ((HoldReasonCode > 5) && (HoldReasonCode < 15))
# (Job is Held AND Error && # Holds >= 3)
periodic_remove = (JobStatus == 5) && ((HoldReasonCode > 5) && (HoldReasonCode < 15)) && (NumShadowStarts >= 3)
queue
"""


   def __makeSerialStagedTemplate(self):
      return """# RemoteBatchFACTORY:makeSerialStagedTemplate
universe = vanilla
executable = APPSCRIPTNAME
transfer_executable = false
environment = "ENVIRONMENT"
output = factory_RUNNAME_INSTANCEID.stdout
error = factory_RUNNAME_INSTANCEID.stderr
log = factory_RUNNAME_INSTANCEID.log
notification = never
+FactoryJob = true
x509userproxy = /tmp/hub-proxy.HUBUSERNAME
+FactoryRefreshProxy = true
periodic_release = (CurrentTime > (EnteredCurrentStatus + 300)) && ((HoldReasonCode > 5) && (HoldReasonCode < 15))
# (Job is Held AND Error && # Holds >= 3)
periodic_remove = (JobStatus == 5) && ((HoldReasonCode > 5) && (HoldReasonCode < 15)) && (NumShadowStarts >= 3)
queue
"""


   def __buildSerialFile(self):
      # setup regex's for the template
      re_runName           = re.compile("RUNNAME")
      re_jobId             = re.compile("JOBID")
      re_instanceId        = re.compile("INSTANCEID")
      re_appScriptName     = re.compile("APPSCRIPTNAME")
      re_walltime          = re.compile("WALLTIME")
      re_environment       = re.compile("ENVIRONMENT")
      re_instanceDirectory = re.compile("INSTANCEDIRECTORY")
      re_hubUserName       = re.compile("HUBUSERNAME")
      re_hubUserId         = re.compile("HUBUSERID")

      if self.stageFiles:
         template = self.__makeSerialStagedTemplate()
      else:
         template = self.__makeSerialNotStagedTemplate()

      template = re_runName.sub(self.runName,template)
      template = re_jobId.sub(self.localJobId,template)
      template = re_instanceId.sub(self.instanceId,template)
      template = re_appScriptName.sub(self.appScriptName,template)
      template = re_walltime.sub(self.wallTime,template)
      template = re_environment.sub(self.environment,template)
      template = re_instanceDirectory.sub(self.instanceDirectory,template)
      template = re_hubUserName.sub(self.hubUserName,template)
      template = re_hubUserId.sub(str(self.hubUserId),template)

      return(template)


   def buildBatchScript(self):
      batchLogName = ""
      batchScriptName = ""
      if self.isMultiCoreRequest:
         if   self.computationMode == 'mpi':
            batchScript = ""
         elif self.computationMode == 'parallel':
            batchScript = ""
         elif self.computationMode == 'matlabmpi':
            batchScript = ""
      else:
         batchLogName = "factory_%s_%s.log" % (self.runName,self.instanceId)
         batchScript = self.__buildSerialFile()
         batchScriptName = "%s_%s.factory" % (self.localJobId,self.instanceId)

      return(batchLogName,batchScriptName,batchScript)


   def getBatchNodeList(self):
      return(self.nodeFileName,self.nodeList)


